@@ -5,19 +5,19 @@ Returns zero or more YAML Paths indicating where in given YAML/Compatible data
 a search expression matches.  Values and/or keys can be searched.  EYAML can be
 employed to search encrypted values.
 
-Copyright 2019, 2020 William W. Kimball, Jr. MBA MSIS
+Copyright 2019, 2020, 2021 William W. Kimball, Jr. MBA MSIS
 """
 import sys
 import argparse
 import json
 from os import access, R_OK
 from os.path import isfile
-from typing import Any, Generator, List, Optional, Tuple
+from typing import Any, Dict, Generator, List, Optional, Tuple
 
-from ruamel.yaml.comments import CommentedSeq, CommentedMap
+from ruamel.yaml.comments import CommentedSeq, CommentedMap, CommentedSet
 
 from yamlpath import __version__ as YAMLPATH_VERSION
-from yamlpath.common import Parsers, Searches
+from yamlpath.common import Anchors, Parsers, Searches
 from yamlpath.exceptions import YAMLPathException
 from yamlpath.enums import (
     AnchorMatches,
@@ -376,7 +376,7 @@ def search_for_paths(logger: ConsolePrinter, processor: EYAMLProcessor,
                      pathsep: PathSeperators = PathSeperators.DOT,
                      build_path: str = "",
                      seen_anchors: Optional[List[str]] = None,
-                     **kwargs: bool) -> Generator[YAMLPath, None, None]:
+                     **kwargs) -> Generator[YAMLPath, None, None]:
     """
     Recursively search a data structure for nodes matching an expression.
 
@@ -391,6 +391,7 @@ def search_for_paths(logger: ConsolePrinter, processor: EYAMLProcessor,
     include_value_aliases: bool = kwargs.pop("include_value_aliases", False)
     decrypt_eyaml: bool = kwargs.pop("decrypt_eyaml", False)
     expand_children: bool = kwargs.pop("expand_children", False)
+    all_anchors: Dict[str, Any] = kwargs.pop("all_anchors", {})
     strsep = str(pathsep)
     invert = terms.inverted
     method = terms.method
@@ -459,8 +460,8 @@ def search_for_paths(logger: ConsolePrinter, processor: EYAMLProcessor,
                         include_key_aliases=include_key_aliases,
                         include_value_aliases=include_value_aliases,
                         decrypt_eyaml=decrypt_eyaml,
-                        expand_children=expand_children
-                ):
+                        expand_children=expand_children,
+                        all_anchors=all_anchors):
                     logger.debug(
                         "Yielding RECURSED match, {}.".format(subpath),
                         prefix="yaml_paths::search_for_paths<list>:  ",
@@ -587,7 +588,7 @@ def search_for_paths(logger: ConsolePrinter, processor: EYAMLProcessor,
                     yield YAMLPath(tmp_path)
                 continue
 
-            if isinstance(val, (CommentedSeq, CommentedMap)):
+            if isinstance(val, (CommentedSeq, CommentedMap, CommentedSet)):
                 logger.debug(
                     "Recursing into complex data:", data=val,
                     prefix="yaml_paths::search_for_paths<dict>:  ",
@@ -600,8 +601,8 @@ def search_for_paths(logger: ConsolePrinter, processor: EYAMLProcessor,
                         include_key_aliases=include_key_aliases,
                         include_value_aliases=include_value_aliases,
                         decrypt_eyaml=decrypt_eyaml,
-                        expand_children=expand_children
-                ):
+                        expand_children=expand_children,
+                        all_anchors=all_anchors):
                     logger.debug(
                         "Yielding RECURSED match, {}.".format(subpath),
                         prefix="yaml_paths::search_for_paths<dict>:  ",
@@ -626,6 +627,67 @@ def search_for_paths(logger: ConsolePrinter, processor: EYAMLProcessor,
                     )
                     yield YAMLPath(tmp_path)
 
+        # Include YAML Merge Keys when include_value_aliases is enabled
+        if include_value_aliases:
+            refs = data.merge if hasattr(data, "merge") else []
+            for (_, ref_node) in refs:
+                for anchor_name, anchor_node in all_anchors.items():
+                    if anchor_node == ref_node:
+                        tmp_path = (build_path + "[&{}]".format(
+                            YAMLPath.escape_path_section(
+                                anchor_name, pathsep)))
+                        matches = Searches.search_matches(
+                            method, term, anchor_name)
+                        if ((matches and not invert)
+                            or (invert and not matches)
+                        ):
+                            logger.debug(
+                                ("yaml_paths::search_for_paths<ymk>:"
+                                + "yielding YMK-VALUE match, {}:  {}."
+                                ).format(anchor_name, tmp_path)
+                            )
+                            yield YAMLPath(tmp_path)
+
+    elif isinstance(data, CommentedSet):
+        if build_path:
+            build_path += strsep
+        elif pathsep is PathSeperators.FSLASH:
+            build_path = strsep
+
+        for key in data:
+            tmp_path = build_path + YAMLPath.escape_path_section(key, pathsep)
+
+            # The key itself may be an Anchor or Alias.  Search it when the
+            # caller wishes.
+            key_anchor_matched = Searches.search_anchor(
+                key, terms, seen_anchors, search_anchors=search_anchors,
+                include_aliases=include_key_aliases)
+            logger.debug(
+                ("yaml_paths::search_for_paths<set>:"
+                    + "KEY anchor search, {}:  {}.")
+                .format(key, key_anchor_matched)
+            )
+
+            if key_anchor_matched in [AnchorMatches.MATCH,
+                                      AnchorMatches.ALIAS_INCLUDED]:
+                logger.debug(
+                    ("yaml_paths::search_for_paths<set>:"
+                        + "yielding a KEY-ANCHOR match, {}."
+                    ).format(key, tmp_path)
+                )
+                yield YAMLPath(tmp_path)
+                continue
+
+            # Search the name of the key, itself
+            matches = Searches.search_matches(method, term, key)
+            if (matches and not invert) or (invert and not matches):
+                logger.debug(
+                    ("yaml_paths::search_for_paths<set>:"
+                        + "yielding KEY name match, {}:  {}."
+                    ).format(key, tmp_path)
+                )
+                yield YAMLPath(tmp_path)
+
 def get_search_term(logger: ConsolePrinter,
                     expression: str) -> Optional[SearchTerms]:
     """
@@ -712,7 +774,7 @@ def print_results(
             # output become messy.
             for node_coordinate in processor.get_nodes(result, mustexist=True):
                 node = node_coordinate.node
-                if isinstance(node, (dict, list)):
+                if isinstance(node, (dict, list, CommentedSet)):
                     resline += "{}".format(
                         json.dumps(Parsers.jsonify_yaml_data(node)))
                 else:
@@ -743,6 +805,8 @@ def process_yaml_file(
 
         # Process all searches
         processor.data = yaml_data
+        all_anchors: Dict[str, Any] = {}
+        Anchors.scan_for_anchors(yaml_data, all_anchors)
         yaml_paths = []
         for expression in args.search:
             exterm = get_search_term(log, expression)
@@ -760,7 +824,8 @@ def process_yaml_file(
                     include_key_aliases=include_key_aliases,
                     include_value_aliases=include_value_aliases,
                     decrypt_eyaml=args.decrypt,
-                    expand_children=args.expand):
+                    expand_children=args.expand,
+                    all_anchors=all_anchors):
                 # Record only unique results
                 add_entry = True
                 for entry in yaml_paths:
@@ -792,7 +857,8 @@ def process_yaml_file(
                         include_key_aliases=include_key_aliases,
                         include_value_aliases=include_value_aliases,
                         decrypt_eyaml=args.decrypt,
-                        expand_children=args.expand):
+                        expand_children=args.expand,
+                        all_anchors=all_anchors):
                     for entry in yaml_paths:
                         if str(result) == str(entry[1]):
                             yaml_paths.remove(entry)
