@@ -6,15 +6,17 @@ pre-defined keywords (in the programming language sense).
 
 Copyright 2020 William W. Kimball, Jr. MBA MSIS
 """
-from typing import Any, Generator, List
+from typing import Any, Dict, Generator, List
+
+from ruamel.yaml.comments import CommentedMap
 
 from yamlpath.types import AncestryEntry, PathSegment
 from yamlpath.enums import PathSearchKeywords, PathSearchMethods
+from yamlpath.common import Anchors, Nodes, Searches
 from yamlpath.path import SearchKeywordTerms
 from yamlpath.exceptions import YAMLPathException
 from yamlpath.wrappers import NodeCoords
 from yamlpath import YAMLPath
-import yamlpath.common
 
 class KeywordSearches:
     """Helper methods for common data searching operations."""
@@ -70,6 +72,55 @@ class KeywordSearches:
     def has_child(
         data: Any, invert: bool, parameters: List[str], yaml_path: YAMLPath,
         **kwargs: Any
+    ) -> Generator[NodeCoords, None, None]:
+        """
+        Indicate whether data has a named or anchored child.
+
+        Parameters:
+        1. data (Any) The data to evaluate
+        2. invert (bool) Invert the evaluation
+        3. parameters (List[str]) Parsed parameters
+        4. yaml_path (YAMLPath) YAML Path begetting this operation
+
+        Keyword Arguments:
+        * parent (ruamel.yaml node) The parent node from which this query
+          originates
+        * parentref (Any) The Index or Key of data within parent
+        * relay_segment (PathSegment) YAML Path segment presently under
+          evaluation
+        * translated_path (YAMLPath) YAML Path indicating precisely which node
+          is being evaluated
+        * ancestry (List[AncestryEntry]) Stack of ancestors preceding the
+          present node under evaluation
+
+        Returns:  (Generator[NodeCoords, None, None]) each result as it is
+            generated
+        """
+        # There must be exactly one parameter
+        param_count = len(parameters)
+        if param_count != 1:
+            raise YAMLPathException(
+                ("Invalid parameter count to {}; {} required, got {} in"
+                 " YAML Path").format(
+                     PathSearchKeywords.HAS_CHILD, 1, param_count),
+                str(yaml_path))
+        match_key = parameters[0]
+
+        if match_key[0] == "&":
+            matches = KeywordSearches._has_anchored_child(
+                data, invert, parameters, yaml_path, **kwargs)
+        else:
+            matches = KeywordSearches._has_concrete_child(
+                data, invert, parameters, yaml_path, **kwargs)
+
+        for match in matches:
+            yield match
+
+    @staticmethod
+    # pylint: disable=locally-disabled,too-many-locals
+    def _has_concrete_child(
+        data: Any, invert: bool, parameters: List[str], yaml_path: YAMLPath,
+        **kwargs: Any
     ) -> Generator[NodeCoords, None, None]:
         """
         Indicate whether data has a named child.
@@ -100,14 +151,6 @@ class KeywordSearches:
         ancestry: List[AncestryEntry] = kwargs.pop("ancestry", [])
         relay_segment: PathSegment = kwargs.pop("relay_segment", None)
 
-        # There must be exactly one parameter
-        param_count = len(parameters)
-        if param_count != 1:
-            raise YAMLPathException(
-                ("Invalid parameter count to {}; {} required, got {} in"
-                 " YAML Path").format(
-                     PathSearchKeywords.HAS_CHILD, 1, param_count),
-                str(yaml_path))
         match_key = parameters[0]
 
         # Against a map, this will return nodes which have an immediate
@@ -130,10 +173,10 @@ class KeywordSearches:
             # Against an AoH, this will scan each element's immediate children,
             # treating and yielding as if this search were performed directly
             # against each map in the list.
-            if yamlpath.common.Nodes.node_is_aoh(data):
+            if Nodes.node_is_aoh(data):
                 for idx, ele in enumerate(data):
                     next_path = translated_path.append("[{}]".format(str(idx)))
-                    for aoh_match in KeywordSearches.has_child(
+                    for aoh_match in KeywordSearches._has_concrete_child(
                         ele, invert, parameters, yaml_path,
                         parent=data, parentref=idx, translated_path=next_path
                     ):
@@ -160,6 +203,124 @@ class KeywordSearches:
                 ("{} data has no child nodes in YAML Path").format(type(data)),
                 str(yaml_path))
 
+    @staticmethod
+    # pylint: disable=locally-disabled,too-many-locals,too-many-branches,too-many-statements
+    def _has_anchored_child(
+        data: Any, invert: bool, parameters: List[str], yaml_path: YAMLPath,
+        **kwargs: Any
+    ) -> Generator[NodeCoords, None, None]:
+        """
+        Indicate whether data has an anchored child.
+
+        Parameters:
+        1. data (Any) The data to evaluate
+        2. invert (bool) Invert the evaluation
+        3. parameters (List[str]) Parsed parameters
+        4. yaml_path (YAMLPath) YAML Path begetting this operation
+
+        Keyword Arguments:
+        * parent (ruamel.yaml node) The parent node from which this query
+          originates
+        * parentref (Any) The Index or Key of data within parent
+        * relay_segment (PathSegment) YAML Path segment presently under
+          evaluation
+        * translated_path (YAMLPath) YAML Path indicating precisely which node
+          is being evaluated
+        * ancestry (List[AncestryEntry]) Stack of ancestors preceding the
+          present node under evaluation
+
+        Returns:  (Generator[NodeCoords, None, None]) each result as it is
+            generated
+        """
+        parent: Any = kwargs.pop("parent", None)
+        parentref: Any = kwargs.pop("parentref", None)
+        translated_path: YAMLPath = kwargs.pop("translated_path", YAMLPath(""))
+        ancestry: List[AncestryEntry] = kwargs.pop("ancestry", [])
+        relay_segment: PathSegment = kwargs.pop("relay_segment", None)
+
+        match_key = parameters[0]
+        anchor_name = match_key[1:] if match_key[0] == "&" else match_key
+
+        if isinstance(data, CommentedMap):
+            # Look for YAML Merge Keys by the Anchor name
+            all_data = ancestry[0][0] if len(ancestry) > 0 else data
+            all_anchors: Dict[str, Any] = {}
+            Anchors.scan_for_anchors(all_data, all_anchors)
+            compare_node = (all_anchors[anchor_name]
+                            if anchor_name in all_anchors
+                            else None)
+            is_ymk_anchor = (
+                compare_node is not None and isinstance(compare_node, dict))
+
+            if is_ymk_anchor:
+                child_present = False
+                if hasattr(data, "merge") and len(data.merge) > 0:
+                    # Ignore comparision if there is no source
+                    for (idx, merge_node) in data.merge:
+                        if merge_node == compare_node:
+                            child_present = True
+                            break
+
+                if (
+                    (invert and not child_present) or
+                    (child_present and not invert)
+                ):
+                    yield NodeCoords(
+                        data, parent, parentref, translated_path,
+                        ancestry, relay_segment)
+                    return
+
+            # Look for Anchored keys; include merged nodes
+            else:
+                child_present = False
+                for (key, val) in data.items():
+                    key_anchor = Anchors.get_node_anchor(key)
+                    val_anchor = Anchors.get_node_anchor(val)
+                    if key_anchor and key_anchor == anchor_name:
+                        child_present = True
+                        break
+                    if val_anchor and val_anchor == anchor_name:
+                        child_present = True
+                        break
+
+                if (
+                    (invert and not child_present) or
+                    (child_present and not invert)
+                ):
+                    yield NodeCoords(
+                        data, parent, parentref, translated_path,
+                        ancestry, relay_segment)
+
+        elif Nodes.node_is_aoh(data, accept_nulls=True):
+            for idx, ele in enumerate(data):
+                if ele is None:
+                    continue
+
+                next_path = translated_path.append("[{}]".format(str(idx)))
+                next_ancestry = ancestry + [(data, idx)]
+                for aoh_match in KeywordSearches._has_anchored_child(
+                    ele, invert, parameters, yaml_path,
+                    parent=data, parentref=idx, translated_path=next_path,
+                    ancestry=next_ancestry
+                ):
+                    yield aoh_match
+
+        elif isinstance(data, list):
+            child_present = False
+            for ele in data:
+                ele_anchor = Anchors.get_node_anchor(ele)
+                if ele_anchor and ele_anchor == anchor_name:
+                    child_present = True
+                    break
+
+            if (
+                (invert and not child_present) or
+                (child_present and not invert)
+            ):
+                yield NodeCoords(
+                    data, parent, parentref, translated_path,
+                    ancestry, relay_segment)
+
     @staticmethod
     # pylint: disable=locally-disabled,too-many-locals
     def name(
@@ -261,7 +422,10 @@ class KeywordSearches:
         match_value: Any = None
         match_nodes: List[NodeCoords] = []
         discard_nodes: List[NodeCoords] = []
-        if yamlpath.common.Nodes.node_is_aoh(data):
+        unwrapped_data: Any = NodeCoords.unwrap_node_coords(data)
+        if Nodes.node_is_aoh(
+            unwrapped_data, accept_nulls=True
+        ):
             # A named child node is mandatory
             if scan_node is None:
                 raise YAMLPathException((
@@ -270,13 +434,14 @@ class KeywordSearches:
                     ).format(PathSearchKeywords.MAX),
                     str(yaml_path))
 
-            for idx, ele in enumerate(data):
+            for idx, wrapped_ele in enumerate(data):
+                ele = NodeCoords.unwrap_node_coords(wrapped_ele)
                 next_path = translated_path + "[{}]".format(idx)
                 next_ancestry = ancestry + [(data, idx)]
-                if scan_node in ele:
+                if ele is not None and scan_node in ele:
                     eval_val = ele[scan_node]
                     if (match_value is None
-                        or yamlpath.common.Searches.search_matches(
+                        or Searches.search_matches(
                             PathSearchMethods.GREATER_THAN, match_value,
                             eval_val)
                     ):
@@ -290,7 +455,7 @@ class KeywordSearches:
                         continue
 
                     if (match_value is None
-                        or yamlpath.common.Searches.search_matches(
+                        or Searches.search_matches(
                             PathSearchMethods.EQUALS, match_value,
                             eval_val)
                     ):
@@ -313,15 +478,15 @@ class KeywordSearches:
                     str(yaml_path))
 
             for key, val in data.items():
+                next_path = (
+                    translated_path + YAMLPath.escape_path_section(
+                        key, translated_path.seperator))
+                next_ancestry = ancestry + [(data, key)]
                 if isinstance(val, dict):
                     if val is not None and scan_node in val:
                         eval_val = val[scan_node]
-                        next_path = (
-                            translated_path + YAMLPath.escape_path_section(
-                                key, translated_path.seperator))
-                        next_ancestry = ancestry + [(data, key)]
                         if (match_value is None
-                            or yamlpath.common.Searches.search_matches(
+                            or Searches.search_matches(
                                 PathSearchMethods.GREATER_THAN, match_value,
                                 eval_val)
                         ):
@@ -335,7 +500,7 @@ class KeywordSearches:
                             continue
 
                         if (match_value is None
-                            or yamlpath.common.Searches.search_matches(
+                            or Searches.search_matches(
                                 PathSearchMethods.EQUALS, match_value,
                                 eval_val)
                         ):
@@ -375,7 +540,7 @@ class KeywordSearches:
                 if (ele is not None
                     and (
                         match_value is None or
-                        yamlpath.common.Searches.search_matches(
+                        Searches.search_matches(
                             PathSearchMethods.GREATER_THAN, match_value,
                             ele)
                 )):
@@ -389,7 +554,7 @@ class KeywordSearches:
                     continue
 
                 if (ele is not None
-                    and yamlpath.common.Searches.search_matches(
+                    and Searches.search_matches(
                         PathSearchMethods.EQUALS, match_value,
                         ele)
                 ):
@@ -464,7 +629,10 @@ class KeywordSearches:
         match_value: Any = None
         match_nodes: List[NodeCoords] = []
         discard_nodes: List[NodeCoords] = []
-        if yamlpath.common.Nodes.node_is_aoh(data):
+        unwrapped_data: Any = NodeCoords.unwrap_node_coords(data)
+        if Nodes.node_is_aoh(
+            unwrapped_data, accept_nulls=True
+        ):
             # A named child node is mandatory
             if scan_node is None:
                 raise YAMLPathException((
@@ -473,13 +641,14 @@ class KeywordSearches:
                     ).format(PathSearchKeywords.MIN),
                     str(yaml_path))
 
-            for idx, ele in enumerate(data):
+            for idx, wrapped_ele in enumerate(data):
+                ele = NodeCoords.unwrap_node_coords(wrapped_ele)
                 next_path = translated_path + "[{}]".format(idx)
                 next_ancestry = ancestry + [(data, idx)]
-                if scan_node in ele:
+                if ele is not None and scan_node in ele:
                     eval_val = ele[scan_node]
                     if (match_value is None
-                        or yamlpath.common.Searches.search_matches(
+                        or Searches.search_matches(
                             PathSearchMethods.LESS_THAN, match_value,
                             eval_val)
                     ):
@@ -493,7 +662,7 @@ class KeywordSearches:
                         continue
 
                     if (match_value is None
-                        or yamlpath.common.Searches.search_matches(
+                        or Searches.search_matches(
                             PathSearchMethods.EQUALS, match_value,
                             eval_val)
                     ):
@@ -516,15 +685,15 @@ class KeywordSearches:
                     str(yaml_path))
 
             for key, val in data.items():
+                next_ancestry = ancestry + [(data, key)]
+                next_path = (
+                    translated_path + YAMLPath.escape_path_section(
+                        key, translated_path.seperator))
                 if isinstance(val, dict):
                     if val is not None and scan_node in val:
                         eval_val = val[scan_node]
-                        next_path = (
-                            translated_path + YAMLPath.escape_path_section(
-                                key, translated_path.seperator))
-                        next_ancestry = ancestry + [(data, key)]
                         if (match_value is None
-                            or yamlpath.common.Searches.search_matches(
+                            or Searches.search_matches(
                                 PathSearchMethods.LESS_THAN, match_value,
                                 eval_val)
                         ):
@@ -538,7 +707,7 @@ class KeywordSearches:
                             continue
 
                         if (match_value is None
-                            or yamlpath.common.Searches.search_matches(
+                            or Searches.search_matches(
                                 PathSearchMethods.EQUALS, match_value,
                                 eval_val)
                         ):
@@ -578,7 +747,7 @@ class KeywordSearches:
                 if (ele is not None
                     and (
                         match_value is None or
-                        yamlpath.common.Searches.search_matches(
+                        Searches.search_matches(
                             PathSearchMethods.LESS_THAN, match_value,
                             ele)
                 )):
@@ -592,7 +761,7 @@ class KeywordSearches:
                     continue
 
                 if (ele is not None
-                    and yamlpath.common.Searches.search_matches(
+                    and Searches.search_matches(
                         PathSearchMethods.EQUALS, match_value,
                         ele)
                 ):
